/* BEGIN HEADER ===============================================================

    Project:            Arcnet Driver for Windows NT
    Programname:        ARCREAD
    Modulename:
    Filename:           arcread.c

    Description:        Simple Example for Controller Access and receive

    Current Version:    1.2
    Created:            21.11.2000

    Author:             S. Menger
    Company:            SoHard AG

    Changes:

    Version Date     ch. by Ch-Idx Ch.-Description
    ------- -------- ------ ------ --------
    1.0     21.11.00 MEN           Creation
    1.1     06.04.04 MEN           Message changed (CTRL-C)
    1.2     04.10.04 MEN           Bugfix (szDriverName size)
	2.0		30.03.17 NEI		   Expanded to ARCNET Mirror

			(c) SoHard AG 1996-2000 All rights reserved
** END HEADER   =============================================================*/

/* ------------------  COMPILER SWITCHES    ------------------*/

/* ------------------      INCLUDES         ------------------*/
#include "windows.h"
#include "stdio.h"
#include "conio.h"
#include "winioctl.h"
#include "stdlib.h"
#include "farc.h"

/* ------------------    DEFINITIONS        ------------------*/

#define VERSION "V1.2"

#define ARCDATAMAX 514
/* ------------------     CONSTANTS         ------------------*/

/* ------------------       TYPES           ------------------*/

/* ------------------ MODULE-GLOBAL VARIABLES  ---------------*/

/* ------------------  EXTERNAL VARIABLES   ------------------*/

/* ------------------  EXTERNAL FUNCTIONENS ------------------*/

void *__gxx_personality_v0;

VOID showUsage(VOID);


   int 
__cdecl main(int argc, char **argv) {

   HANDLE      hDriver;
   DWORD       dwErrorCode;
   ULONG       dwInterruptCount;
   DWORD       dwBytesReceived, dwBytesWritten;
   DWORD       dwReadTimeout;
   ARCNET_DCB  dcb;
   CHAR        szDriverName[256] = "\\\\.\\" ;
   BOOL        zLeaveLoop;
   CHAR        szVersion[26];
   BYTE        packet[514], bSwap;
   
   int		   verbose = 0;

   printf("ARCREAD " VERSION ", SOHARD ARCNET driver read-sample\n\n");
   if ( argc < 3 )  {
      showUsage();
      return 1;
   }

   if ( (dcb.b_node_id = (unsigned char)atoi( argv[2] )) == 0) { // NodeID from 1st Argument
      showUsage();
      return 1;
   }
 
   if (argc >= 4) {
	if ( argv[3][0] == 'v' || argv[3][0] == 'V' ) {
	   verbose = 1;
	}
   }
   
   printf("\nverbose %i      argc %o\n", verbose, argc);
   
   strcat(szDriverName, argv[1] );

   if ((hDriver = CreateFile(
      szDriverName,                    // Name
      GENERIC_READ | GENERIC_WRITE,     // Access
      0,                                // ShareMode
      NULL,                             // Security_Attributes
      OPEN_EXISTING,                    // Create
      FILE_ATTRIBUTE_NORMAL,            // File Attributes
      NULL                              // Template
      )) != INVALID_HANDLE_VALUE ) {

      printf( "Driver opened!\n" );

      // initialize Controller

      // set DCB-Structure
      dcb.z_et_1 = TRUE;
      dcb.z_et_2 = TRUE;
      dcb.z_et_3 = FALSE;
      dcb.z_ckp_1 = FALSE;
      dcb.z_ckp_2 = FALSE;
      dcb.z_ckp_3 = FALSE;
      dcb.z_backplane = FALSE;
      dcb.z_p1mode = FALSE;
      dcb.z_four_naks = FALSE;
      dcb.b_init_behaviour = eARC_INIT_FORCE; //[1]

      if (DeviceIoControl(
         hDriver,           // Handle
         IOCTL_FARC_INIT,   // Controlcode
         &dcb,              // Input-Buffer
         sizeof(dcb),       // Bufferlength
         NULL,//buffer,              // Output-Buffer
         0, //sizeof(buffer),                 // Bufferlength
         &dwBytesReceived,  // Bytes read
         NULL               // Pointer to Overlapped Struktur, here not used
         ) == FALSE) {

         dwErrorCode = GetLastError();

         if (dwErrorCode == E_FARC_INIT_FORCED) {
            printf( "No activity/token found. Controller-Init forced.\n"
                    "This maybe a quite normal situation, if you have no\n"
                    "other active node in the network.\n\n");
         }
         else {
            printf( "Controller-Init fails. Errorcode %lu\n", dwErrorCode );
         }
      }
      else {
         dwErrorCode = 0;
         printf( "Controller-Init success.\n" );
      }
      if ((dwErrorCode == E_FARC_INIT_FORCED) || (dwErrorCode == 0))
      {

         dwReadTimeout = 1000000;
         if (DeviceIoControl(
            hDriver,
            IOCTL_FARC_SET_READ_TIMEOUT,
            &dwReadTimeout,
            sizeof( ULONG ),
            NULL,
            0,
            &dwBytesReceived,
            NULL
            ) == TRUE ) {
            printf("Try to start ReadFile with 1000 seconds read timeout...\n\n");
         }
         else {
            printf("Try to start ReadFile. Waiting infinitely for ARCNET data...\n\n");
         }
         // wait for datapacket

         zLeaveLoop = FALSE;
         do {
            if ( ReadFile(
               hDriver,
               packet,
               ARCDATAMAX,
               &dwBytesReceived,
               NULL
               ) == FALSE) {

               dwErrorCode = GetLastError();
               if (dwErrorCode == E_FARC_EXCESSIVE_RECON) {
                  printf("E_FARC_EXCESSIVE_RECON\n");
                  zLeaveLoop = FALSE;
               }
               else {
                  printf( "Read fails. Errorcode %lu\n", dwErrorCode );
                  zLeaveLoop = FALSE;
               }
            }
            else {
				int i;

				printf("\nPacket received. SID: %3u   DID: %3u   Size Payload: %3u", packet[0], packet[1], dwBytesReceived-6 );

                if (verbose) {
					printf("\nIN:  ");
					for (i=6; i<dwBytesReceived; i++) {
						printf("%02X", (int) packet[i]);
						printf(((i-5)%25)?" ":"\n     ");
					}
				}
				
// Child's play:  Modifying the content of the ARCNET payload data packages

				for (i=6; i<dwBytesReceived; i++) { // Loop over all payload bytes

				  // packet[i]++;  // inkrements the content of each data byte
				  
				  switch (packet[i]) { // "State-Machine" adressing the 4 digital output LEDs of the Phoenix PLC
					case 1:
						packet[i] = 2;
						break;
					case 2:
	                    packet[i] = 8;
						break;
					case 8:
	                    packet[i] = 4;
						break;
					default:
					case 4:
	                    packet[i] = 1;
						break;
				  }
				}
// End of Child's play
				
				if (verbose) {
					printf("\nOUT: ");
					for (i=6; i<dwBytesReceived; i++) {
						printf("%02X", (int) packet[i]);
						printf(((i-5)%25)?" ":"\n     ");
					}
					printf("\n");
				}
							
				bSwap = packet[0]; packet[0] = packet[1]; packet[1] = bSwap;  //    swap SID and DID
				
			    if ( WriteFile(	hDriver, packet, dwBytesReceived, &dwBytesWritten, NULL ) == FALSE) {

					dwErrorCode = GetLastError();
					printf( "Send fails. Errorcode %lu\n", dwErrorCode );

				}
				
                zLeaveLoop = FALSE;
            }
         } while (!zLeaveLoop);

         if ( DeviceIoControl(
            hDriver,
            IOCTL_FARC_GET_INTERRUPTS,
            NULL,
            0,
            &dwInterruptCount,
            sizeof( ULONG ),
            &dwBytesReceived,
            NULL
            ) == TRUE ) {

            printf( "Interrupt counter: %lu\n", dwInterruptCount );
            printf( "Note, that RECON-Events causes also interrupts!\n\n" );
         }

         if ( DeviceIoControl(
            hDriver,
            IOCTL_FARC_GET_SWVERSION,
            NULL,
            0,
            szVersion,
            sizeof( szVersion ),
            &dwBytesReceived,
            NULL
            ) == TRUE ) {

            printf( "Driver version: %s\n", szVersion );
         }
         else {
           dwErrorCode = GetLastError();
           printf( "Version read fails. Errorcode: %lu\n", dwErrorCode );
         }
         
         DeviceIoControl(
            hDriver,
            IOCTL_FARC_DEINIT,
            NULL,
            0,
            NULL,
            0,
            &dwBytesReceived,
            NULL
            );
      }
      CloseHandle( hDriver );
   } 
   else {
       dwErrorCode = GetLastError();
       printf("Can't get a handle to Driver %s. Errorcode: %lu\n",argv[1],dwErrorCode);
   }

   // if program not started in console window
   printf("\n\nPress any key...\n");
   do ; while (!kbhit());
   getch();
   return 0;

}


VOID showUsage(VOID)
{
   printf("Usage: arcread DriverAccessName NodeID [verbose]\n"
          "  e.g. arcread FARC 5\n"
          "Note, that this program will not return,"
          "until the controller received an arcnet packet!"
          "\nPress any key...\n"
          );
   do ; while (!kbhit());
   getch();
}
